/* $Id: ast_forstatement.cpp 6310 2023-01-16 20:39:48Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "ast_forstatement.h"
#include "ast_assignment.h"
#include "ast_expression.h"
#include "ast_compound.h"
#include "../astvisitor.h"

wtast::ForStatement::~ForStatement()
{
	delete mInit;
	delete mCond;
	delete mStep;
	delete mBody;
}

void wtast::ForStatement::Accept(AstVisitor* vis)
{
	if(vis->Visit(this))
	{
		if(mInit) { mInit->Accept(vis); }
		if(mCond) { mCond->Accept(vis); }
		if(mStep) { mStep->Accept(vis); }
		mBody->Accept(vis);

		vis->EndVisit(this);
	}
}

wtast::ForStatement* wtast::ForStatement::Copy()
{
	Assignment* init = 0;
	Expression* cond = 0;
	Assignment* step = 0;

	if(mInit) { init = mInit->Copy(); }
	if(mCond) { cond = mCond->Copy(); }
	if(mStep) { step = mStep->Copy(); }

	return new ForStatement(init, cond, step, mBody->Copy());
}
