/* $Id: ast_functionsignature.h 6310 2023-01-16 20:39:48Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef WTPARSER_AST_FUNCSIG_H
#define WTPARSER_AST_FUNCSIG_H

#include <string>
#include <deque>
#include "../astnode.h"
#include "../uniqueid.h"

namespace wtparser
{
class ParserHelper;
} // wtparser

namespace wtast
{

class Type;
class TypeInfo;
class FunctionArgument;

class FunctionSignature : public AstNode
{
	typedef std::deque<FunctionArgument*> args_t;
	typedef std::deque<Type*> types_t;
	std::string        mName;
	args_t             mArgs;
	types_t            mRets;
	bool               mPure;
	bool               mBuiltin;
	bool               mGenerated;
	wtparser::UniqueId mUniqueId;

public:
	FunctionSignature(const std::string & name, bool pure)
		: AstNode()
		, mName(name)
		, mPure(pure)
		, mBuiltin(false)
		, mGenerated(false)
		, mUniqueId(0)
	{
		// ok
	}

	~FunctionSignature();

	void Accept(AstVisitor* vis);

	FunctionSignature* Copy();

	Kind GetKind() const
	{
		return FUNCTIONSIGNATURE;
	}

	std::string & GetName()
	{
		return mName;
	}

	std::string GetCanonicalName(bool argnames, bool rets);

	void AddArgFront(FunctionArgument* arg)
	{
		mArgs.push_front(arg);
	}

	void AddArgBack(FunctionArgument* arg)
	{
		mArgs.push_back(arg);
	}

	void AddRetFront(Type* type)
	{
		mRets.push_front(type);
	}

	void AddRetBack(Type* type)
	{
		mRets.push_back(type);
	}

	TypeInfo* GetArgType();

	std::size_t GetArgc() const
	{
		return mArgs.size();
	}

	FunctionArgument* GetArg(std::size_t idx)
	{
		if(idx<mArgs.size())
		{
			return mArgs[idx];
		}
		else
		{
			return 0;
		}
	}

	std::size_t GetRetc() const
	{
		return mRets.size();
	}

	Type* GetRet(std::size_t idx)
	{
		if(idx<mRets.size())
		{
			return mRets[idx];
		}
		else
		{
			return 0;
		}
	}

	TypeInfo* GetRetType();

	bool IsPure() const
	{
		return mPure;
	}

	bool IsBuiltin() const
	{
		return mBuiltin;
	}

	bool IsGenerated() const
	{
		return mGenerated;
	}

	void SetUniqueId(wtparser::UniqueId unique_id)
	{
		mUniqueId = unique_id;
	}

	wtparser::UniqueId GetUniqueId() const
	{
		return mUniqueId;
	}

	friend class wtparser::ParserHelper;

private:

	void SetBuiltin()
	{
		mBuiltin = true;
	}

	void SetGenerated()
	{
		mGenerated = true;
	}
};

} // namespace wtast

#endif // WTPARSER_AST_FUNCSIG_H
