/* $Id: ast_simultaneoustype.h 6310 2023-01-16 20:39:48Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef WTPARSER_AST_SIMULTANEOUSTYPE_H
#define WTPARSER_AST_SIMULTANEOUSTYPE_H

#include "../astnode.h"
#include "ast_typeinfo.h"
#include <deque>

namespace wtast
{

class Type;

class SimultaneousType : public TypeInfo
{
	typedef std::deque<Type*> types_t;

	types_t     mTypes;
	std::string mName;

public:
	SimultaneousType(Type* t1=0, Type* t2=0)
		: TypeInfo()
	{
		if(t1) { AddType(t1); }
		if(t2) { AddType(t2); }
	}

	~SimultaneousType();

	void Accept(AstVisitor* vis);

	SimultaneousType* Copy();

	Kind GetKind() const
	{
		return SIMULTANEOUSTYPE;
	}

	void AddType(Type* t)
	{
		mTypes.push_back(t);
	}

	virtual std::size_t GetNumberOfTypes() const // from wtast::TypeInfo
	{
		return mTypes.size();
	}

	virtual Type* GetType(std::size_t idx) // from wtast::TypeInfo
	{
		if(idx<mTypes.size())
		{
			return mTypes[idx];
		}
		else
		{
			return 0;
		}
	}

	std::string GetName() const;

	virtual bool HasObject() const;

	virtual bool HasFunctionType() const;

	virtual bool IsSimultaneous() const { return true; } // from wtast::TypeInfo

	virtual bool IsCopyable() const; // from wtast::TypeInfo
};

} // namespace wtast

#endif // WTPARSER_AST_SIMULTANEOUSTYPE_H
