//================================================================================================
// $Id: sourcemanager.cpp 6309 2023-01-16 20:19:42Z aron $
//
// Project:
//		Welltype Parser
// Contents:
//		Source manager
// Authors:
//		Aron Barath, 2014-2023
//================================================================================================
// Copyright (c) Aron Barath
//================================================================================================

#include "sourcemanager.h"
#include "lex.yy.cc.h"

namespace wtparser
{

void SourceManager::FileEntry::EraseContent()
{
	delete lexer;
	delete file;
}

std::string SourceManager::_MakeTopPath(const std::string & fname)
{
	if(mFileStack.empty())
	{
		return fname;
	}

	const std::string & path = mFileStack.back().path;

	if(path.empty())
	{
		return fname;
	}

	return path + "/" + fname;
}

IFile* SourceManager::_OpenFile(const std::string & filename, std::string & path)
{
	std::string fullname = _MakeTopPath(filename);
	IFile* file = mISourceManager->OpenFile(fullname);

	for(includes_t::const_iterator it=mIncludeDirs.begin(),end=mIncludeDirs.end();!file && it!=end;++it)
	{
		fullname = *it + "/" + filename;
		file = mISourceManager->OpenFile(fullname);
	}

	if(file)
	{
		std::size_t pos = fullname.rfind('/');

		if(pos==std::string::npos)
		{
			pos = fullname.rfind('\\');
		}

		if(pos!=std::string::npos)
		{
			path = fullname.substr(0, pos);
		}
		else
		{
			path = "";
		}
	}

	return file;
}

SourceManager::Result SourceManager::OpenFile(const std::string & filename)
{
	if(mFileStack.size()<DepthLimit)
	{
		std::string path;
		IFile* file = _OpenFile(filename, path);

		if(file)
		{
			WTFlexLexer* lexer = new WTFlexLexer(mHelper, file, mTopLexer);

			if(lexer)
			{
				mFileStack.push_back(FileEntry(filename, path, file, lexer));
				mTopLexer = lexer;

				return OK;
			}
			else
			{
				delete file;
				return OUTOFMEMORY;
			}
		}
		else
		{
			return FILEOPENERROR;
		}
	}
	else
	{
		return DEPTHLIMITEXCEEDED;
	}
}

bool SourceManager::CloseTop()
{
	if(!mFileStack.empty())
	{
		mTopLexer = _Top().lexer->GetPrevious();
		_Top().EraseContent();
		mFileStack.pop_back();

		return true;
	}

	return false;
}

} // namespace wtparser

