//================================================================================================
// $Id: sourcemanager.h 6309 2023-01-16 20:19:42Z aron $
//
// Project:
//		Welltype Parser
// Contents:
//		Source manager
// Authors:
//		Aron Barath, 2014-2023
//================================================================================================
// Copyright (c) Aron Barath
//================================================================================================

#ifndef WTPARSER_SOURCEMANAGER_H
#define WTPARSER_SOURCEMANAGER_H

#include <string>
#include <vector>
#include "frontend.h"

class WTFlexLexer;

namespace wtparser
{

class ParserHelper;

class SourceManager
{
	struct FileEntry
	{
		FileEntry(const std::string & n, const std::string & p, IFile* f, WTFlexLexer* l)
			: name(n)
			, path(p)
			, file(f)
			, lexer(l)
		{
			// ok
		}

		void EraseContent();

		std::string  name;
		std::string  path;
		IFile*       file;
		WTFlexLexer* lexer;
	};

	typedef std::vector<FileEntry>   filestack_t;
	typedef std::vector<std::string> includes_t;

	FileEntry & _Top()
	{
		return mFileStack.back();
	}

	std::string _MakeTopPath(const std::string & fname);

	IFile* _OpenFile(const std::string & filename, std::string & path);

	ParserHelper*   mHelper;
	ISourceManager* mISourceManager;
	WTFlexLexer*    mTopLexer;
	filestack_t     mFileStack;
	includes_t      mIncludeDirs;

public:
	SourceManager(ParserHelper* helper, ISourceManager* ism)
		: mHelper(helper)
		, mISourceManager(ism)
		, mTopLexer(0)
	{
		// ok
	}

	~SourceManager()
	{
		while(CloseTop()) { /* ok */ }
	}

	static const unsigned int DepthLimit = 200;

	enum Result
	{
		OK,
		OUTOFMEMORY,
		FILEOPENERROR,
		DEPTHLIMITEXCEEDED
	};

public:
	Result OpenFile(const std::string & filename);
	bool CloseTop();

	WTFlexLexer* GetTopLexer()
	{
		return mTopLexer;
	}

	bool IsEmpty() const
	{
		return mFileStack.empty();
	}

	std::string GetFilename(bool with_path)
	{
		if(!mFileStack.empty())
		{
			FileEntry & top = _Top();

			if(with_path)
			{
				return top.path + "/" + top.name;
			}

			return top.name;
		}
		else
		{
			return "";
		}
	}

	void PushIncludeDir(const std::string & dir)
	{
		mIncludeDirs.push_back(dir);
	}
};

} // namespace wtparser

#endif // WTPARSER_SOURCEMANAGER_H

