#include "config_private.h"
#include "function.h"
#include "profile.h"
#include "funcsig.h"
#include "program.h"

struct wtr_function* wtr_function_new(struct wtr_profile* profile, struct wtr_funcsig* sig,
	struct wtr_cfunction* cfunc, struct wtr_program* prog, wt_u32 pc)
{
	struct wtr_function* fh;

	if((cfunc && prog) || (!cfunc && !prog))
	{
		return NULL;
	}

	if(NULL!=wtr_profile_findfunction(profile, sig))
	{
		return NULL;
	}

	if(NULL!=(fh=(struct wtr_function*)wtr_profile_sys_malloc(profile, sizeof(struct wtr_function))))
	{
		fh->profile  = profile;
		fh->program  = prog;
		fh->cfunc    = cfunc;
		fh->funcsig  = sig;
		fh->refcount = 0;
		fh->hancount = 0;
		fh->pfn._pad = 0;

		if(NULL!=cfunc)
		{
			/* artificially increase the reference counter for C functions, */
			/* because these functions shall not be implicitly deleted */
			++fh->refcount;

			fh->pfn.ptr = cfunc;
			fh->pfn.ext = profile->external_tag;
		}
		else
		{
			fh->pfn.ptr = prog;
			fh->pfn.ext = prog->pc_base + pc;
		}

		wtr_list_insert_tail(&profile->list_functions, &fh->link_functions);
	}

	return fh;
}

void wtr_function_release(struct wtr_profile* profile, struct wtr_function* func)
{
	if(NULL!=func && (0==func->refcount || 0==(--func->refcount)))
	{
		wtr_list_remove(&func->link_functions);
		wtr_function_remhandle(profile, func);
		wtr_funcsig_delete(profile, func->funcsig);
		wtr_cfunction_delete(profile, func->cfunc);
		wtr_profile_sys_free(profile, func);
	}
}

wt_function wtr_function_genhandle(struct wtr_profile* profile, struct wtr_function* func)
{
	if(NULL!=func)
	{
		if(0==func->hancount)
		{
			++func->refcount;

			if(NULL!=func->program)
			{
				++func->profile->refcount;
			}
		}

		++func->hancount;
	}

	return func;
}

void wtr_function_remhandle(struct wtr_profile* profile, wt_function func)
{
	if(NULL!=func && (0==func->hancount || 0==(--func->hancount)))
	{
		if(NULL!=func->program)
		{
			--func->profile->refcount;
		}
	}
}

struct wtr_cfunction* wtr_cfunction_new(struct wtr_profile* profile, struct wtr_funcsig* sig, WT_PFN_EXTFUNC func, void* token)
{
	struct wtr_cfunction* const cf = (struct wtr_cfunction*)wtr_profile_sys_malloc(profile, sizeof(struct wtr_cfunction));

	if(NULL!=cf)
	{
		cf->profile     = profile;
		cf->funcsig     = sig;
		cf->token       = token;
		cf->extfunc_pfn = func;

		++profile->refcount;
	}

	return cf;
}

void wtr_cfunction_delete(struct wtr_profile* profile, struct wtr_cfunction* cfunc)
{
	if(NULL!=cfunc)
	{
		--profile->refcount;

		wtr_profile_sys_free(profile, cfunc);
	}
}
